#-----------------------------------------------------------------------------
# Project  Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File     atpg.tcl
# Author   Erika Miller-Jolicoeur <erika.miller-jolicoeur@polymtl.ca>
# Lab      GRM - Polytechnique Montreal
# Date     <2020-07-23>
#-----------------------------------------------------------------------------
# Brief    Génération automatique des vecteurs de test
# Usage    cd implementation
#          modus -files ../scripts/atpg.tcl
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_SYN   1;  # Netlist post-synthèse
set DO_PNR   0;  # Netlist post-placement-routage

set DESIGN compteur;

if { !($DO_SYN ^ $DO_PNR) } { error "ERREUR: Mauvaise configuration" }

if { ![info exists FLOT]             } { 
    if { $DO_SYN } { set FLOT  syn }
    if { $DO_PNR } { set FLOT  pnr }}
if { ![info exists DO_ALL_FROM_MAKE] } { set DO_ALL_FROM_MAKE  0         }
if { ![info exists DO_EXIT]          } { set DO_EXIT           0         }

switch $FLOT {
    syn {
	set workD $::env(ATPG_DIR)/syn
	set net $::env(SYN_DFT_NET_DIR)/${DESIGN}.syn}
    pnr {   
	set workD $::env(ATPG_DIR)/pnr
	set net $::env(PNR_DFT_NET_DIR)/${DESIGN}.route
    }
    default {
        error "ERREUR: Sélectionnez un flot valide parmis: syn et pnr"
    }
}
file mkdir ${workD}
#-------------------------------------------------------------------------------
# INITIALIZE VARIABLES
#-------------------------------------------------------------------------------
 
# "set_db workdir" is recommended to be set first to initialize where the Database is to be located
# WORKDIR Tcl variable can be used in the script to reference file locations
set    WORKDIR ${workD};
set_db workdir $WORKDIR;                                  # Specify the directory for the ATPG database
 
# Configure the database
set_option stdout     all;                                # Only print a summary for each command to the terminal
                                                          # Logs contain the complete command output
set ::env(CDS_LIC_REPORT) yes
# test_checks.tcl provides check_log proc to stop script on message severity
source  $::env(Install_Dir)/bin/64bit/test_checks.tcl;    # Load test_checks.tcl from Modus installation - provides check_log below
set STOP_ON_MSG_SEV ERROR;                                # Messagee severity for check_log to stop at (ERROR, WARNING_SEVERE, WARNING)
set LOGDIR          $WORKDIR/testresults/logs;            # Location of logs for check_log
 
file delete -force $WORKDIR/tbdata;                       # Delete Test Database
file delete -force $WORKDIR/testresults;                  # Delete Test Output Files/Logs
 
#-------------------------------------------------------------------------------
# BUILD THE LOGIC MODEL
#-------------------------------------------------------------------------------
build_model \
    -cell                ${DESIGN} \
    -techlib             $::env(FE_VER_LIB)/*.v \
    -designsource        ${net}.v \
    -blackboxoutputs     z  \
    -allowmissingmodules no \

check_log log_build_model
 
# Report
report_model_statistics
#-------------------------------------------------------------------------------
# BUILD THE TEST MODEL
#-------------------------------------------------------------------------------
build_testmode \
    -testmode	 FULLSCAN \
    -assignfile	 $::env(ATPG_DIR)/${DESIGN}.FULLSCAN.pinassign \
    -modedef	 FULLSCAN \
    -delaymode   zero \
 
check_log log_build_testmode_FULLSCAN

#-------------------------------------------------------------------------------
# VERIFY THE TEST MODEL
#-------------------------------------------------------------------------------
verify_test_structures \
    -testmode	    FULLSCAN \
    -xclockanalysis yes \
    -testxsource    yes \
 
check_log log_verify_test_structures_FULLSCAN
  
#-------------------------------------------------------------------------------
# REPORT THE TEST MODEL
#-------------------------------------------------------------------------------
report_test_structures \
    -testmode	     FULLSCAN \
    -reportscanchain all \
 
check_log log_report_test_structures_FULLSCAN
 
#-------------------------------------------------------------------------------
# BUILD THE FAULT MODEL
#-------------------------------------------------------------------------------
build_faultmodel \
    -includedynamic	 yes \
 
check_log log_build_faultmodel

# Reports
report_faults \
    -faulttype   all \
    -reportkey   yes

report_fault_statistics

#-------------------------------------------------------------------------------
# ATPG - TEST GENERATION
#-------------------------------------------------------------------------------
# Analyse préliminaire fautes statiques
analyze_testability \
    -testmode FULLSCAN \

# Tests fautes scan + collé à 0/1
create_logic_tests \
    -experiment	${DESIGN}_atpg \
    -testmode	FULLSCAN \

check_log log_create_logic_tests_FULLSCAN_${DESIGN}_atpg

report_faults \
    -testmode   FULLSCAN \
    -experiment ${DESIGN}_atpg \
    -faulttype  static \
    -reportkey  yes \

# Tests pour sensibilisation fautes avec séquence
create_sequential_tests \
    -testmode    FULLSCAN \
    -experiment  ${DESIGN}_atpg \
    -append      yes \

check_log log_create_sequential_tests_FULLSCAN_${DESIGN}_atpg

# Tests pour fautes dynamiques
create_logic_delay_tests \
    -testmode   FULLSCAN \
    -experiment ${DESIGN}_atpg \
    -append     yes \

check_log log_create_logic_delay_tests_FULLSCAN_${DESIGN}_atpg

# Tests pour courants de repos
create_iddq_tests \
    -testmode   FULLSCAN \
    -experiment ${DESIGN}_atpg \
    -append     yes \

check_log log_create_iddq_tests_FULLSCAN_${DESIGN}_atpg

# Reports
report_fault_statistics \
    -experiment ${DESIGN}_atpg \
    -testmode   FULLSCAN \

report_faults \
    -testmode   FULLSCAN \
    -experiment ${DESIGN}_atpg \
    -faulttype  all \
    -reportkey  yes \

#-------------------------------------------------------------------------------
# ATPG - Report the Scan and Capture Switching
#-------------------------------------------------------------------------------
write_toggle_gram \
    -experiment	 ${DESIGN}_atpg \
    -testmode	 FULLSCAN \

#-------------------------------------------------------------------------------
# ATPG - Save Experiment to the Master Database for the Testmode
#-------------------------------------------------------------------------------
commit_tests \
    -inexperiment  ${DESIGN}_atpg \
    -testmode	   FULLSCAN 

check_log log_commit_tests_FULLSCAN_${DESIGN}_atpg

#-------------------------------------------------------------------------------
# VERILOG VECTORS - For PARALLEL Simulation
#-------------------------------------------------------------------------------
write_vectors \
    -testmode	 FULLSCAN \
    -language	 verilog \
    -scanformat	 parallel \
    -exportdir   ${WORKDIR}/testresults/verilog/FULLSCAN_PARALLEL \
    -logfile     ${WORKDIR}/testresults/logs/log_write_vectors_FULLSCAN_${DESIGN}_atpg

check_log log_write_vectors_FULLSCAN_${DESIGN}_atpg

#-----------------------------------------------------------------------------
# SORTIE
#-----------------------------------------------------------------------------
if { $DO_EXIT } {
    exit
}
