#-----------------------------------------------------------------------------
# Project  Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File     pnr.tcl
# Author   Mickael Fiorentino  <mickael.fiorentino@polymtl.ca>
# Lab      GRM - Polytechnique Montreal
# Date     <2019-09-10 Tue>
#-----------------------------------------------------------------------------
# Brief    Placement & Routage du mini-mips avec Innovus
# Usage    cd implementation
#          innovus -file ../scripts/pnr.tcl
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_INIT    1;  # Initialisation des librairies
set DO_FP      1;  # Floorplan
set DO_POWER   1;  # Alimentations
set DO_IO      1;  # Entrées / Sorties
set DO_PLACE   1;  # Placement
set DO_CTS     1;  # Arbre d'horloge
set DO_ROUTE   1;  # Routage
set DO_SAVE    1;  # Sauvegarde

set DESIGN compteur

set flot dft;  # base, cg, dft

# Use values from Makefile if defined, else use script defaults values
if { ![info exists FLOT]             } { set FLOT              ${flot}   }
if { ![info exists DO_ALL_FROM_MAKE] } { set DO_ALL_FROM_MAKE  0         }
if { ![info exists DO_EXIT]          } { set DO_EXIT           0         }

switch $FLOT {
    base {
	set WITH_DFT     0;  # Options DFT

	# Définition des répertoires de destination
        set DIR_SYN_NET $::env(SYN_NET_DIR)
	set DIR_PNR_NET $::env(PNR_NET_DIR)
	set DIR_REP     $::env(PNR_REP_DIR)
	set DIR_LOG     $::env(PNR_LOG_DIR)
    }
    cg {
	set WITH_DFT     0;  # Options DFT

	# Définition des répertoires de destination
        set DIR_SYN_NET $::env(SYN_CG_NET_DIR)
	set DIR_PNR_NET $::env(PNR_CG_NET_DIR)
	set DIR_REP     $::env(PNR_CG_REP_DIR)
	set DIR_LOG     $::env(PNR_CG_LOG_DIR)
    }
    dft {
	set WITH_DFT    1;  # Options DFT

	# Définition des répertoires de destination
        set DIR_SYN_NET $::env(SYN_DFT_NET_DIR)
	set DIR_PNR_NET $::env(PNR_DFT_NET_DIR)
	set DIR_REP     $::env(PNR_DFT_REP_DIR)
	set DIR_LOG     $::env(PNR_DFT_LOG_DIR)

    }
    default {
	error "ERREUR: Sélectionnez un flot valide parmis: base, cg et dft"
    }
}

# Répertoires de destination
file mkdir ${DIR_REP} ${DIR_PNR_NET} ${DIR_LOG}
set OPT_DIR $::env(PNR_DIR)/${FLOT}_opt
set TIM_DIR ${DIR_REP}/timing
set OA_DIR  ${FLOT}_compteur_oa

#-----------------------------------------------------------------------------
# INIT
#-----------------------------------------------------------------------------
if { $DO_INIT || $DO_ALL_FROM_MAKE } {

    set init_oa_ref_lib    [list gsclib045_tech gsclib045 giolib045]
    set init_verilog       ${DIR_SYN_NET}/${DESIGN}.syn.v
    set init_design_settop 1
    set init_top_cell      $DESIGN
    set init_gnd_net       VSS
    set init_pwr_net       VDD
    set init_mmmc_file     $::env(CONST_DIR)/mmmc.tcl

    init_design
}

#-----------------------------------------------------------------------------
# FLOORPLAN
#-----------------------------------------------------------------------------
if { $DO_FP || $DO_ALL_FROM_MAKE } {

    floorPlan -site CoreSite -r 0.9 0.6 1 1 1 1
}

#-----------------------------------------------------------------------------
# ALIMENTATIONS
#-----------------------------------------------------------------------------
if { $DO_POWER || $DO_ALL_FROM_MAKE } {

    globalNetConnect VDD -type pgpin -pin VDD -inst * -override
    globalNetConnect VSS -type pgpin -pin VSS -inst * -override
    globalNetConnect VDD -type tiehi -inst * -override
    globalNetConnect VSS -type tielo -inst * -override

    addStripe -nets VDD           \
              -layer Metal1       \
              -direction vertical \
              -width 0.6          \
              -number_of_sets 1   \
              -start_from left    \
              -start_offset -0.8

    addStripe -nets VSS           \
              -layer Metal1       \
              -direction vertical \
              -width 0.6          \
              -number_of_sets 1   \
              -start_from right   \
              -start_offset -0.8

    sroute -nets { VDD VSS }                   \
           -connect { corePin floatingStripe } \
           -layerChangeRange { Metal1(1) Metal1(1) }
}

#-----------------------------------------------------------------------------
# I/O
#-----------------------------------------------------------------------------
if { $DO_IO || $DO_ALL_FROM_MAKE } {

    set top_nets [list    [get_nets -quiet "i_clk"]  \
                          [get_nets -quiet "i_rstn"] \
		          [get_nets -quiet "i_en"]]
    set bottom_nets [list [get_nets -quiet "o_cnt[*]"]]

    if { $FLOT eq "dft" } {

	set left_nets [list   [get_nets -quiet "i_scan_en"]   \
			      [get_nets -quiet "i_tdi"]]
	set right_nets [list  [get_nets -quiet "i_test_mode"] \
                              [get_nets -quiet "o_tdo"]]
    }
    setPinAssignMode -pinEditInBatch true

    set top_n [list ]
    foreach nets $top_nets {foreach_in_collection n $nets {lappend top_n [get_object_name $n]}}
    editPin -pin "${top_n}" -edge 1 -layer 4 -spreadType SIDE \
        -offsetEnd 2 -offsetStart 2 -spreadDirection clockwise \
        -pinWidth 0.06 -pinDepth 0.335 -fixOverlap 1

    set bottom_n [list ]
    foreach nets $bottom_nets {foreach_in_collection n $nets {lappend bottom_n [get_object_name $n]}}
    editPin -pin "${bottom_n}" -edge 3 -layer 4 -spreadType SIDE \
        -offsetEnd 2 -offsetStart 2 -spreadDirection counterclockwise \
        -pinWidth 0.06 -pinDepth 0.335 -fixOverlap 1

    if { $FLOT eq "dft" } {

	set left_n [list ]
	foreach nets $left_nets {foreach_in_collection n $nets {lappend left_n [get_object_name $n]}}
	editPin -pin "${left_n}" -edge 0 -layer 3 -spreadType SIDE \
	    -offsetEnd 2 -offsetStart 2 -spreadDirection clockwise \
	    -pinWidth 0.06 -pinDepth 0.335 -fixOverlap 1

	set right_n [list ]
	foreach nets $right_nets {foreach_in_collection n $nets {lappend right_n [get_object_name $n]}}
	editPin -pin "${right_n}" -edge 2 -layer 3 -spreadType SIDE \
	    -offsetEnd 2 -offsetStart 2 -spreadDirection clockwise \
	    -pinWidth 0.06 -pinDepth 0.335 -fixOverlap 1
    }
}

#-----------------------------------------------------------------------------
# PLACEMENT
#-----------------------------------------------------------------------------
if { $DO_PLACE || $DO_ALL_FROM_MAKE } {
    
    # reset signal
    #set_interactive_constraint_modes [all_constraint_modes]
    #reset_ideal_network [get_nets rstn_sync\[1\]]

    # Timing
    timeDesign -prePlace -outDir ${TIM_DIR}

    # Settings
    setDesignMode -process 45 -flowEffort standard
    setPlaceMode -timingDriven true  \
                 -place_global_cong_effort auto

    if { $FLOT eq "dft" } {
	setPlaceMode -place_global_reorder_scan true

	# Scan Chain
	defIn ${DIR_SYN_NET}/${DESIGN}.syn.scandef
	setScanReorderMode -compLogic true
    } else {
	setPlaceMode -place_global_reorder_scan false
	deleteAllScanCells
    }

    # Placement
    place_opt_design -out_dir ${OPT_DIR}
    checkPlace

    # Netlist
    saveNetlist ${DIR_PNR_NET}/${DESIGN}.place.v
    write_sdf -version 2.1 -target_application verilog -interconn noport \
        ${DIR_PNR_NET}/${DESIGN}.place.sdf

    if { $FLOT eq "dft" } {
	# Write Scan Chain Order
	defOutBySection ${DIR_PNR_NET}/${DESIGN}.place.scandef -noNets -noComps -scanChains 
    }
}

#-----------------------------------------------------------------------------
# ARBRE D'HORLOGE
#-----------------------------------------------------------------------------
if { $DO_CTS || $DO_ALL_FROM_MAKE } {

    set_ccopt_property buffer_cells \
        [list CLKBUFX20 CLKBUFX16 CLKBUFX12 CLKBUFX8 CLKBUFX6 CLKBUFX4 CLKBUFX3 CLKBUFX2]

    set_ccopt_property inverter_cells \
        [list CLKINVX20 CLKINVX6 CLKINVX8 CLKINVX16 CLKINVX12 CLKINVX4 CLKINVX3 CLKINVX2 CLKINVX1]

    set_ccopt_property use_inverters true

    set_ccopt_property clock_gating_cells TLATNTSCA*

    # Synthèse de l'arbre d'horloge
    ccopt_design -cts -outDir ${OPT_DIR}
    optDesign -postCTS -outDir ${OPT_DIR}

    # Timing
    timeDesign -postCTS -outDir ${TIM_DIR}
    timeDesign -hold -postCTS -outDir ${TIM_DIR}

    # Netlist
    saveNetlist ${DIR_PNR_NET}/${DESIGN}.cts.v
    write_sdf -version 2.1 -target_application verilog -interconn noport \
        ${DIR_PNR_NET}/${DESIGN}.cts.sdf
}

#-----------------------------------------------------------------------------
# ROUTE
#-----------------------------------------------------------------------------
if { $DO_ROUTE || $DO_ALL_FROM_MAKE } {

    # Routage timing- et signal-integrity- driven
    setNanoRouteMode -routeWithTimingDriven true
    setNanoRouteMode -routeWithSIDriven true
    routeDesign -GlobalDetail

    # Extraction des capacités parasites
    setExtractRCMode -engine postRoute
    extractRC

    # Optimisations post-route
    setAnalysisMode -analysisType onChipVariation
    setAnalysisMode -cppr both
    optDesign -postRoute -setup -hold -outDir ${OPT_DIR}

    # Ajout des filler cells
    addFiller -cell FILL32 FILL16 FILL8 FILL4 FILL2 FILL1 -prefix FILLER

    # Verifications physiques
    set_verify_drc_mode -report ${DIR_REP}/${DESIGN}.drc.rpt
    verify_drc
    verifyConnectivity -type all -error 1000 -warning 50 -report ${DIR_REP}/${DESIGN}.con.rpt

    # Timing
    timeDesign -postRoute -outDir ${TIM_DIR}
    report_timing > ${DIR_REP}/${DESIGN}.timing.rpt

    # Netlist
    saveNetlist ${DIR_PNR_NET}/${DESIGN}.route.v
    write_sdf -version 2.1 -target_application verilog -interconn noport \
        ${DIR_PNR_NET}/${DESIGN}.route.sdf
}

#-----------------------------------------------------------------------------
# SAUVEGARDES
#-----------------------------------------------------------------------------
if { $DO_SAVE || $DO_ALL_FROM_MAKE } {

    cd $::env(PNR_DIR)

    if {! [file isdirectory $OA_DIR] } {
        createLib $OA_DIR -attachTech gsclib045_tech
    }
    saveDesign -cellview "$OA_DIR $DESIGN layout"

    cd ../
}
#-----------------------------------------------------------------------------
# SORTIE
#-----------------------------------------------------------------------------
if { $DO_EXIT } {
    exit
}
