#-----------------------------------------------------------------------------
# Project  Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File     pwr.tcl
# Authors  Mickael Fiorentino <mickael.fiorentino@polymtl.ca>
# Lab      GRM - Polytechnique Montréal
# Date     <2019-10-02 Wed>
#-----------------------------------------------------------------------------
# Brief    Estimation du puissance du compteur BCD avec Voltus
# Usage    source setup.csh && cd implementation
#          voltus -files ../scripts/pwr.tcl
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_INIT  1
set DO_STAT  1
set DO_DYN   1

# Configurations par défaut
set flot        dft;  # base, cg, dft

set DESIGN     compteur
set DUT        compteur_tb/dut/m_compteur
set PERIOD     1250ps
set RESOLUTION 100ps
set VCD_START  0ps
set VCD_END    54375ps


# Use values from Makefile if defined, else use script defaults values
if { ![info exists FLOT]             } { set FLOT              ${flot}   }
if { ![info exists DO_ALL_FROM_MAKE] } { set DO_ALL_FROM_MAKE  0         }
if { ![info exists DO_EXIT]          } { set DO_EXIT           0         }

# Définition des répertoires
switch $FLOT {
    base    { set DIR_REP $::env(PNR_REP_DIR)     }
    cg      { set DIR_REP $::env(PNR_CG_REP_DIR)  }
    dft     { set DIR_REP $::env(PNR_DFT_REP_DIR) }
    default { error "ERREUR: Sélectionnez un flot valide parmis: base, cg et dft" }
}

set OA ${FLOT}_compteur_oa

#-----------------------------------------------------------------------------
# INIT
#-----------------------------------------------------------------------------
if { $DO_INIT || $DO_ALL_FROM_MAKE } {

    cd $::env(PNR_DIR)

    # Unités par défaut
    set_time_unit -picoseconds

    # Importation du layout
    read_design -cellview "$OA $DESIGN layout" -physical_data

    # Mise à jour du timing à partir du mmmc
    set_analysis_view -update_timing

    # Configuration
    set_pg_library_mode -extraction_tech_file $::env(BE_QRC_LIB)/gpdk045.tch -celltype techonly \
                        -power_pins {VDD 1.1} -ground_pins VSS                                  \
                        -decap_cells DECAP* -filler_cells FILL*                                 \

    generate_pg_library -output ${DIR_REP}/power
    cd ../
}

#-----------------------------------------------------------------------------
# ANALYSE STATIQUE
#-----------------------------------------------------------------------------
if { $DO_STAT || $DO_ALL_FROM_MAKE } {

    set rep ${DESIGN}.pwr.stat

    # Configuration statique
    set_power_analysis_mode -reset
    set_power_analysis_mode -method static               \
                            -corner max                  \
                            -create_binary_db false      \
                            -write_static_currents false

    # Activité moyenne
    set_default_switching_activity -reset
    set_default_switching_activity -input_activity 0.2 -period $PERIOD

    # Report power
    report_power -format detailed -report_prefix $rep -output ${DIR_REP}/power
    file copy -force ${DIR_REP}/power/${rep}.rpt ${DIR_REP}/${rep}.rpt
}

#-----------------------------------------------------------------------------
# ANALYSE DYNAMIQUE
#-----------------------------------------------------------------------------
if { $DO_DYN || $DO_ALL_FROM_MAKE } {

    set rep ${DESIGN}.pwr.dyn

    # Configuration dynamique
    set_power_analysis_mode -reset
    set_power_analysis_mode -method dynamic_vectorbased                               \
                            -enable_rtl_vectorbased_dynamic_analysis true             \
	                    -power_grid_library ${DIR_REP}/power/techonly.cl          \
                            -corner max                                               \
                            -report_stat true                                         \
                            -create_binary_db true                                    \
                            -write_static_currents true                               \
                            -report_missing_flop_outputs false

    # Activité issue du fichier VCD
    set_default_switching_activity -reset
    read_activity_file -reset
    read_activity_file -format VCD $::env(SIM_DIR)/pnr_${FLOT}/${DESIGN}.vcd \
                       -scope  $DUT -start $VCD_START -end $VCD_END

    set_dynamic_power_simulation -resolution $RESOLUTION

    # Report power
    report_power -format detailed -report_prefix $rep -output ${DIR_REP}/power
    file copy -force ${DIR_REP}/power/${rep}.rpt ${DIR_REP}/${rep}.rpt
}
#-----------------------------------------------------------------------------
# SORTIE
#-----------------------------------------------------------------------------
if { $DO_EXIT } {
    exit
}
