#-----------------------------------------------------------------------------
# Project    : Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File       : sim.tcl
# Authors    : Mickael Fiorentino <mickael.fiorentino@polymtl.ca>
# Lab        : GRM - Polytechnique Montréal
# Date       : <2020-08-14>
#-----------------------------------------------------------------------------
# Description: Script pour les simulations du compteur BCD avec Modelsim
# HOW-TO     : source setup.csh && cd simulation/
#            : (CLI) > vsim -c -do ${SCRIPTS_DIR}/sim.tcl
#            : (GUI) > vsim -do ${SCRIPTS_DIR}/sim.tcl
#            : (view) > vsim -view <step>/<top>.wlf -do <step>/<top>.wave.do
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_BEH   0;  # Modèle comportemental
set DO_SYN   0;  # Netlist post-synthèse
set DO_PNR   1;  # Netlist post-placement-routage
set DO_SIM   1;  # 0=Compilation; 1=Compilation + Simulation
set DO_VCD   1;  # Sauvegarde de l'activité au format VCD

set DO_NET_BASE 0;
set DO_NET_CG   0;
set DO_NET_DFT  1;

# set DFT_NET  1; # Pins supplémentaires pour DFT présent dans la netlist  

# if { $DFT_NET } {
#     set config c_dft; # Nom de la config avec chaine à balayage
# }
# if { !$DFT_NET } {
#     set config c_rtl; # Nom de la config sans chaine à balayage
# }
set top    compteur
set tb     compteur_tb
set dut    dut/m_compteur
set work   work
set lib    gpdk45
set wrap   top


# Extract simulation arguments "+<plusarg>"
# +STEP
if { [regexp -- {\+STEP=([^\s]+)?} $argv] } {
    regexp -- {\+STEP=([^\s]+)?} $argv -> STEP
} else {
    if { $DO_BEH } { set FLOT beh }
    if { $DO_SYN } { set FLOT syn }
    if { $DO_PNR } { set FLOT pnr }
}
# +FLOT
if { [regexp -- {\+FLOT=([^\s]+)?} $argv] } {
    regexp -- {\+FLOT=([^\s]+)?} $argv -> FLOT
} else {
    if { $DO_BASE } { set FLOT base }
    if { $DO_CG   } { set FLOT cg   }
    if { $DO_DFT  } { set FLOT dft  }
}
# +VCD
if { [regexp -- {\+VCD=(1)} $argv] } {
    regexp -- {\+VCD=(1)} $argv -> DO_VCD
} 
# +DO_ALL_FROM_MAKE
if { [regexp -- {\+DO_ALL_FROM_MAKE=(1)} $argv] } {
    regexp -- {\+DO_ALL_FROM_MAKE=(1)} $argv -> DO_SIM
}


switch $STEP {
    beh {
	set workD $::env(SIM_DIR)/beh 
	set config c_rtl }
    syn { 
	set workD $::env(SIM_DIR)/syn_${FLOT}
	switch $FLOT {
	    base { 
		set net $::env(SYN_NET_DIR)/${top}.syn 
		set config c_rtl }
	    cg   { 
		set net $::env(SYN_CG_NET_DIR)/${top}.syn
		set config c_rtl }
	    dft  { 
		set net $::env(SYN_DFT_NET_DIR)/${top}.syn 
		set config c_dft }
	    default { 
		error "ERREUR: Sélectionnez un flot valide parmis base, cg, dft" }}}
    pnr { 
	set workD $::env(SIM_DIR)/pnr_${FLOT} 
	switch $FLOT {
	    base { 
		set net $::env(PNR_NET_DIR)/${top}.route 
		set config c_rtl }
	    cg   { 
		set net $::env(PNR_CG_NET_DIR)/${top}.route 
		set config c_rtl }
	    dft  { 
		set net $::env(PNR_DFT_NET_DIR)/${top}.route 
		set config c_dft }
	    default {
		error "ERREUR: Sélectionnez un flot valide parmis base, cg, dft" }}}
    default { 
	error "ERREUR: Sélectionnez une étape valide parmis beh, syn, pnr" }
}

# if { $DO_BEH } { set workD $::env(SIM_DIR)/beh }
# if { $DO_SYN } { set workD $::env(SIM_DIR)/syn }
# if { $DO_PNR } { set workD $::env(SIM_DIR)/pnr }

set log  ${workD}/${top}.log
set wlf  ${workD}/${top}.wlf
set vcd  ${workD}/${top}.vcd
set wave ${workD}/${top}.wave.do

# if { $DO_SYN } { set net $::env(SYN_NET_DIR)/${top}.syn   }
# if { $DO_PNR } { set net $::env(PNR_NET_DIR)/${top}.route }

#-----------------------------------------------------------------------------
# COMPILATION
#-----------------------------------------------------------------------------

# Création du répertoire de travail
if { ![file exist $workD] } {
    file mkdir $workD
}

# Mise à jour du modelsim.ini avec les librairies GPDK45
if { [file exist "./modelsim.ini"] } {
    file delete "./modelsim.ini"
}
vmap -c
vmap $lib $::env(KIT_SIMLIB)/gsclib045_slow

# Work
if { [file exist ${workD}/${work}] } {
    vdel -all -lib ${workD}/${work}
}
vlib ${workD}/${work}
vmap ${work} ${workD}/${work}

# Model
if { $STEP eq "beh" } {
    vcom -2008 -work $work $::env(SRC_DIR)/${top}.vhd

# Netlist
} elseif { $STEP eq "syn" || $STEP eq "pnr" } {
    vlog -work $work ${net}.v
}
vcom -2008 -work $work $::env(SRC_DIR)/${wrap}.vhd
vcom -2008 -work $work $::env(SRC_DIR)/${tb}.vhd

#-----------------------------------------------------------------------------
# SIMULATION
#-----------------------------------------------------------------------------

if { $DO_SIM && $STEP eq "beh" } {
    vsim -c -t ps -voptargs=+acc -do run.do -logfile $log -wlf $wlf ${work}.${config}
}

if { $DO_SIM && ($STEP eq "syn" || $STEP eq "pnr") } {
    vsim -c -t ps -voptargs=+acc -do run.do -logfile $log -wlf $wlf -L $lib -sdfmax ${dut}=${net}.sdf ${work}.${config}
}
