#-----------------------------------------------------------------------------
# Project  Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File     syn.dft.tcl
# Author   Mickael Fiorentino  <mickael.fiorentino@polymtl.ca>
# Lab      GRM - Polytechnique Montreal
# Date     <2020-08-19>
#-----------------------------------------------------------------------------
# Brief    Synthèse du compteur avec Genus
# Usage    cd implementation
#          genus -files ../scripts/syn.dft.tcl
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_INIT     1;  # Initialisation des librairies
set DO_ELAB     1;  # Compilation + élaboration
set DO_SDC      1;  # Contraintes
set SET_SYN     1;  # Options synthese
set DO_SYN      1;  # Synthese
set DO_REPORTS  1;  # Rapports

# Configurations par défaut
set flot        dft;  # base, cg, dft

set DESIGN     compteur;             # Top-level
set SOURCES    [list compteur.vhd];  # Fichiers sources
set SYN_EFFORT high;                 # low | medium | high | express

# Use values from Makefile if defined, else use script defaults values
if { ![info exists FLOT]             } { set FLOT              ${flot}   }
if { ![info exists DO_ALL_FROM_MAKE] } { set DO_ALL_FROM_MAKE  0         }
if { ![info exists DO_EXIT]          } { set DO_EXIT           0         }

switch $FLOT {
    base {
	set ADD_CG      0;  # Ajouter le Clock Gating
	set ADD_DFT     0;  # Options DFT

	# Définition des répertoires de destination
	set DIR_NET $::env(SYN_NET_DIR)
	set DIR_REP $::env(SYN_REP_DIR)
	set DIR_LOG $::env(SYN_LOG_DIR)
    }
    cg {
	set ADD_CG      1;  # Ajouter le Clock Gating
	set ADD_DFT     0;  # Options DFT

	# Définition des répertoires de destination
	set DIR_NET $::env(SYN_CG_NET_DIR)
	set DIR_REP $::env(SYN_CG_REP_DIR)
	set DIR_LOG $::env(SYN_CG_LOG_DIR)
    }
    dft {
	set ADD_CG      1;  # Ajouter le Clock Gating
	set ADD_DFT     1;  # Options DFT

	# Définition des répertoires de destination
	set DIR_NET $::env(SYN_DFT_NET_DIR)
	set DIR_REP $::env(SYN_DFT_REP_DIR)
	set DIR_LOG $::env(SYN_DFT_LOG_DIR)
    }
    default {
	error "ERREUR: Sélectionnez un flot valide parmis: base, cg et dft"
    }
}
file mkdir $DIR_NET $DIR_REP $DIR_LOG
set_db stdout_log $DIR_LOG/genus.log
#-----------------------------------------------------------------------------
# SAVE
#-----------------------------------------------------------------------------
proc save { dir basename } {
    report_timing > ${dir}/${basename}.timing.rpt
    report_area   > ${dir}/${basename}.area.rpt
    report_gates  > ${dir}/${basename}.gates.rpt
    report_power  > ${dir}/${basename}.power.rpt
}

#-----------------------------------------------------------------------------
# INIT
#-----------------------------------------------------------------------------
if { $DO_INIT || $DO_ALL_FROM_MAKE } {

    # Paramètres
    set_db information_level     11;   # Verbosité: 1-11
    set_db hdl_vhdl_read_version 2008; # Standard VHDL
    set_db hdl_error_on_blackbox true; # Éviter les black-box
    set_db hdl_error_on_latch    true; # Éviter les latch
    set_db max_cpus_per_server   8;    # Nombre max de thread utilisable

    # Sources
    set_db init_hdl_search_path [list $::env(SRC_DIR)]
    # Librairies: Timing + Standard Cells + Interconnect
    set_db init_lib_search_path [list $::env(FE_TIM_LIB) \
                                      $::env(BE_LEF_LIB) \
                                      $::env(BE_QRC_LIB)]

    # Librairies de timing: max=setup, min=hold
    read_libs -max_libs slow_vdd1v0_basicCells.lib -min_libs fast_vdd1v0_basicCells.lib

    # Libraries physiques: stdcells + interconnect
    read_physical -lef gsclib045_tech.lef
    read_qrc gpdk045.tch
    set_db interconnect_mode ple; # wireload | ple (Physical Layout Estimators)

    if { $ADD_CG } {
        # Clock gating
        set_db lp_insert_clock_gating true
    }
}

#-----------------------------------------------------------------------------
# ELABORATION
#-----------------------------------------------------------------------------
if { $DO_ELAB || $DO_ALL_FROM_MAKE } {

    # Compilation
    read_hdl -vhdl $SOURCES

    # Élaboration
    elaborate $DESIGN

    # Rapports
    check_design -all > ${DIR_REP}/${DESIGN}.elab.check.rpt
    report_module -depth 3 > ${DIR_REP}/${DESIGN}.elab.hier.rpt
    write_hdl > ${DIR_NET}/${DESIGN}.elab.v
}

#-----------------------------------------------------------------------------
# CONTRAINTES
#-----------------------------------------------------------------------------
if { $DO_SDC || $FROM_MAKE } {

    # Contraintes de timing
    read_sdc $::env(CONST_DIR)/timing.sdc

    # Rapports
    report_timing -lint -verbose > ${DIR_REP}/${DESIGN}.timing_lint.rpt
    report_clocks > ${DIR_REP}/${DESIGN}.clk.rpt
    report_clocks -generated >> ${DIR_REP}/${DESIGN}.clk.rpt
}
#-----------------------------------------------------------------------------
# OPTIONS DE SYNTHÈSE
#-----------------------------------------------------------------------------
if { $SET_SYN || $DO_ALL_FROM_MAKE } {

    # Aplanir la hiérarchie
#    ungroup -all -simple

    #-------------------------------------------------------------------------
    # CONFIGURATION DFT
    #-------------------------------------------------------------------------
    if { $ADD_DFT } {

	# Bascule à balayage
	set_db dft_scan_style muxed_scan

	# Signaux de test
	define_shift_enable -active high -create_port i_scan_en
	define_test_mode -active high -create_port i_test_mode
	define_test_clock -name clk_scan -domain dom_1 -period 20000 i_clk

	# Registres à décalage fonctionnels
	set_db dft_auto_identify_shift_register true
	set_db dft_shift_register_max_length 32

	# Options des bascules à balayage
	set_db design:$DESIGN .dft_scan_map_mode tdrc_pass
	set_db design:$DESIGN .dft_scan_output_preference auto
	set_db design:$DESIGN .dft_connect_scan_data_pins_during_mapping loopback
	set_db design:$DESIGN .dft_connect_shift_enable_during_mapping tie_off

	# Prefixe
	set_db dft_prefix DFT_

	if { $ADD_CG } {
	    # Contrôle du clock gating lors du test
	    set_db design:$DESIGN .lp_clock_gating_test_signal i_scan_en
	}

	# Rapports
	report_scan_setup
	report_scan_setup > ${DIR_REP}/${DESIGN}.dft_conf.scan_setup.rpt

	# Vérification des bascules
	check_dft_rules

	# Résolution des violations
	fix_dft_violations -violations vid_0_async -test_control i_scan_en

	# Rapports
	check_dft_rules > ${DIR_REP}/${DESIGN}.dft_conf.rules.rpt
	report_dft_violations > ${DIR_REP}/${DESIGN}.dft_conf.violations.rpt
    }
}
#-----------------------------------------------------------------------------
# SYNTHÈSE
#-----------------------------------------------------------------------------
if { $DO_SYN || $DO_ALL_FROM_MAKE } {

    # Synthèse générique
    set_db syn_generic_effort $SYN_EFFORT
    syn_generic ${DESIGN}
    save ${DIR_REP} ${DESIGN}.gen

    # Map
    set_db syn_map_effort $SYN_EFFORT
    syn_map ${DESIGN}
    save ${DIR_REP} ${DESIGN}.map
    
    if { $ADD_DFT } {
	# Définition chaîne à balayage
	define_scan_chain -name chain1 -create_ports -sdi i_tdi -sdo o_tdo
	set_db design:${DESIGN} .dft_min_number_of_scan_chains 1
	set_db design:${DESIGN} .dft_mix_clock_edges_in_scan_chains true
	set_db design:${DESIGN} .dft_max_length_of_scan_chains 10
    
	# Connexion des bascules
	connect_scan_chains -auto_create_chains -preview
	connect_scan_chains -auto_create_chains
    
	# Contraintes timing signaux de test
	read_sdc $::env(CONST_DIR)/dft_fullscan.non_scan_mode.sdc
    }

    # Optimisations
    set_db syn_opt_effort $SYN_EFFORT
    syn_opt ${DESIGN}
    save ${DIR_REP} ${DESIGN}.syn
}
#-----------------------------------------------------------------------------
# RAPPORTS
#-----------------------------------------------------------------------------
if { $DO_REPORTS || $DO_ALL_FROM_MAKE } {

    # Netlist
    write_sdc > $::env(CONST_DIR)/${DESIGN}.syn.sdc
    write_hdl > ${DIR_NET}/${DESIGN}.syn.v
    write_sdf -nonegchecks -setuphold split -version 2.1 > ${DIR_NET}/${DESIGN}.syn.sdf

    # DFT files
    if { $ADD_DFT } {
	write_scandef            > ${DIR_NET}/${DESIGN}.syn.scandef
	write_dft_atpg ${DESIGN} -library $::env(FE_VER_LIB)/*.v \
	    -directory $::env(ATPG_DIR)/syn
	file copy -force $::env(ATPG_DIR)/syn/${DESIGN}.FULLSCAN.pinassign $::env(ATPG_DIR)
    }

    # Rapports généraux supplémentaires
    check_design -all     > ${DIR_REP}/${DESIGN}.syn.check.rpt

    # Rapports clock gating
    if { $ADD_CG } {
	report_clock_gating   > ${DIR_REP}/${DESIGN}.syn_cg.clock_gating.rpt
    }

    # Rapports DFT
    if { $ADD_DFT } {
	check_dft_rules       > ${DIR_REP}/${DESIGN}.syn_dft.rules.rpt
	report_dft_violations > ${DIR_REP}/${DESIGN}.syn_dft.violations.rpt
	report_scan_registers > ${DIR_REP}/${DESIGN}.syn_dft.scan_registers.rpt
	report_scan_setup     > ${DIR_REP}/${DESIGN}.syn_dft.scan_setup.rpt
	report_scan_chains    > ${DIR_REP}/${DESIGN}.syn_dft.scan_chains.rpt
	report_scan_registers > ${DIR_REP}/${DESIGN}.syn_dft.scan_registers.rpt
    }

    # Sommaire
    if { $ADD_DFT } {
	report_scan_chains -summary
    }
    report_qor

}
#-----------------------------------------------------------------------------
# SORTIE
#-----------------------------------------------------------------------------
if { $DO_EXIT } {
    exit
}
