#-----------------------------------------------------------------------------
# Project  Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File     syn.tcl
# Author   Mickael Fiorentino  <mickael.fiorentino@polymtl.ca>
# Lab      GRM - Polytechnique Montreal
# Date     <2019-10-01 Tue>
#-----------------------------------------------------------------------------
# Brief    Synthèse du compteur avec Genus
# Usage    cd implementation
#          genus -files ../scripts/syn.tcl
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_INIT     1;  # Initialisation des librairies
set DO_ELAB     1;  # Compilation + élaboration
set DO_SDC      1;  # Contraintes
set DO_SYN      1;  # Synthese
set DO_REPORTS  1;  # Rapports

set DESIGN     compteur;   # Top-level
set SYN_EFFORT high;       # low | medium | high | express

# Create directories if they do not exists
file mkdir $::env(SYN_REP_DIR) $::env(SYN_NET_DIR)

#-----------------------------------------------------------------------------
# SAVE
#-----------------------------------------------------------------------------
proc save { basename } {
    report_timing > $::env(SYN_REP_DIR)/${basename}.timing.rpt
    report_area   > $::env(SYN_REP_DIR)/${basename}.area.rpt
    report_gates  > $::env(SYN_REP_DIR)/${basename}.gates.rpt
    report_power  > $::env(SYN_REP_DIR)/${basename}.power.rpt
}

#-----------------------------------------------------------------------------
# INIT
#-----------------------------------------------------------------------------
if { $DO_INIT } {

    # Paramètres
    set_db information_level     11;   # Verbosité: 1-11
    set_db hdl_vhdl_read_version 2008; # Standard VHDL
    set_db hdl_error_on_blackbox true; # Éviter les black-box
    set_db hdl_error_on_latch    true; # Éviter les latch
    set_db max_cpus_per_server   8;    # Nombre max de thread utilisable

    set_db init_hdl_search_path [list $::env(SRC_DIR)]
    set_db init_lib_search_path [list $::env(FE_TIM_LIB) \
                                      $::env(BE_LEF_LIB) \
                                      $::env(BE_QRC_LIB)]

    # Librairies de timing: max=setup, min=hold
    read_libs -max_libs slow_vdd1v0_basicCells.lib -min_libs fast_vdd1v0_basicCells.lib

    # Libraries physiques: stdcells + interconnect
    read_physical -lef gsclib045_tech.lef
    read_qrc gpdk045.tch
    set_db interconnect_mode ple; # wireload | ple (Physical Layout Estimators)
}

#-----------------------------------------------------------------------------
# ELABORATION
#-----------------------------------------------------------------------------
if { $DO_ELAB } {

    # Compilation
    read_hdl -vhdl [list compteur.vhd]

    # Élaboration
    elaborate $DESIGN

    # Rapports
    check_design -all > $::env(SYN_REP_DIR)/${DESIGN}.elab.check.rpt
    report_module -depth 3 > $::env(SYN_REP_DIR)/${DESIGN}.elab.hier.rpt
    write_hdl > $::env(SYN_NET_DIR)/${DESIGN}.elab.v
}

#-----------------------------------------------------------------------------
# CONTRAINTES
#-----------------------------------------------------------------------------
if { $DO_SDC } {

    # Contraintes de timing
    read_sdc $::env(CONST_DIR)/timing.sdc

    # Rapports
    report_timing -lint -verbose > $::env(SYN_REP_DIR)/${DESIGN}.timing_lint.rpt
    report_clocks > $::env(SYN_REP_DIR)/${DESIGN}.clk.rpt
    report_clocks -generated >> $::env(SYN_REP_DIR)/${DESIGN}.clk.rpt
}

#-----------------------------------------------------------------------------
# SYNTHÈSE
#-----------------------------------------------------------------------------
if { $DO_SYN } {

    # Aplanir la hiérarchie - facultatif 
#    ungroup -all -simple

    # Synthèse générique
    set_db syn_generic_effort $SYN_EFFORT
    syn_generic ${DESIGN}
    save ${DESIGN}.gen

    # Map
    set_db syn_map_effort $SYN_EFFORT
    syn_map ${DESIGN}
    save ${DESIGN}.map

    # Optimisations
    set_db syn_opt_effort $SYN_EFFORT
    syn_opt ${DESIGN}
    save ${DESIGN}.syn

    # Netlist
    write_sdc > $::env(CONST_DIR)/${DESIGN}.syn.sdc
    write_hdl > $::env(SYN_NET_DIR)/${DESIGN}.syn.v
    write_sdf -nonegchecks -setuphold split -version 2.1 > $::env(SYN_NET_DIR)/${DESIGN}.syn.sdf
}
